/* 
   IM Caf - A Java Based Instant Messaging System
   Author: Bruno Freitas - 08/2004
   Contact: bootsector@ig.com.br
 
   Copyright (c) 2004 Bruno Freitas

   Permission is hereby granted, free of charge, to any person obtaining a copy of
   this software and associated documentation files (the "Software"), to deal in 
   the Software without restriction, including without limitation the rights to 
   use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies 
   of the Software, and to permit persons to whom the Software is furnished to do 
   so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in all 
   copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE 
   SOFTWARE.
*/

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

public class AddNewUserDialog extends JDialog {
   private JLabel lblNick, lblFName, lblLName, lblEmail, lblPasswd, lblCPasswd;
   private JTextField txtNick, txtFName, txtLName, txtEmail;
   private JPasswordField txtPasswd, txtCPasswd;
   private JPanel dataPanel, btnsPanel;
   private JButton btnOk, btnCancel;
   private ServerConnection serverConn;
   
   public AddNewUserDialog(Frame owner, ServerConnection serverConn) {
      super(owner, "Register a new User", true);
      
      lblNick = new JLabel("Nickname:");
      lblFName = new JLabel("First Name:");
      lblLName = new JLabel("Last Name:");
      lblEmail = new JLabel("Email:");
      lblPasswd = new JLabel("Password:");
      lblCPasswd = new JLabel("Confirm Password:");
      
      txtNick = new JTextField();
      txtFName = new JTextField();
      txtLName = new JTextField();
      txtEmail = new JTextField();
      txtPasswd = new JPasswordField();
      txtCPasswd = new JPasswordField();
      
      dataPanel = new JPanel(new GridLayout(12,1));
      
      dataPanel.add(lblNick);
      dataPanel.add(txtNick);
      dataPanel.add(lblFName);
      dataPanel.add(txtFName);
      dataPanel.add(lblLName);
      dataPanel.add(txtLName);
      dataPanel.add(lblEmail);
      dataPanel.add(txtEmail);
      dataPanel.add(lblPasswd);
      dataPanel.add(txtPasswd);
      dataPanel.add(lblCPasswd);
      dataPanel.add(txtCPasswd);
      
      btnsPanel = new JPanel(new GridLayout(1, 2));
      
      EvntListener handler = new EvntListener();
      
      btnOk = new JButton("Add");
      btnOk.setMnemonic('A');
      btnOk.setIcon(new ImageIcon(getClass().getResource("res/img/confirm.png")));
      btnOk.addActionListener(handler);
      btnCancel = new JButton("Close");
      btnCancel.setMnemonic('C');
      btnCancel.setIcon(new ImageIcon(getClass().getResource("res/img/closewindow.png")));
      btnCancel.addActionListener(handler);
      
      btnsPanel.add(btnOk);
      btnsPanel.add(btnCancel);
      
      getContentPane().add(dataPanel, BorderLayout.NORTH);
      getContentPane().add(btnsPanel, BorderLayout.SOUTH);
      
      getRootPane().setDefaultButton(btnOk);
      
      setSize(195, 330);
      setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
      setLocationRelativeTo(owner);
      setResizable(false);
      
      this.serverConn = serverConn;
   }
   
   private class EvntListener implements ActionListener {
      
      public void actionPerformed(ActionEvent e) {
         String passwd, cpasswd, info, infoChecker;
         CommData data;
         
         if (e.getSource() == btnOk) {
            if (txtNick.getText().length() == 0) {
               JOptionPane.showMessageDialog(AddNewUserDialog.this, "You must specify a Nickname!");
               txtNick.requestFocus();
               return;
            }
            
            passwd = new String(txtPasswd.getPassword());
            
            if (passwd.length() == 0) {
               JOptionPane.showMessageDialog(AddNewUserDialog.this, "You must specify a Password!");
               txtPasswd.requestFocus();
               return;
            }
            
            cpasswd = new String(txtCPasswd.getPassword());
            
            if (passwd.compareTo(cpasswd) != 0) {
               JOptionPane.showMessageDialog(AddNewUserDialog.this, "The typed passwords are not equal!");
               txtPasswd.requestFocus();
               return;
            }
            
            infoChecker = txtNick.getText() + txtFName.getText() + txtLName.getText() + txtEmail.getText() + passwd;
            
            if (infoChecker.indexOf(':') != -1) {
               JOptionPane.showMessageDialog(AddNewUserDialog.this, "Invalid character ':' was found inside some info you entered. Please correct that!");
               return;
            }

            if (infoChecker.indexOf(',') != -1) {
               JOptionPane.showMessageDialog(AddNewUserDialog.this, "Invalid character ',' was found inside some info you entered. Please correct that!");
               return;
            }
            
            info = txtNick.getText() + ":" +
            ((txtFName.getText().length() == 0)?" ":txtFName.getText()) + ":" +
            ((txtLName.getText().length() == 0)?" ":txtLName.getText()) + ":" +
            ((txtEmail.getText().length() == 0)?" ":txtEmail.getText()) + ":" +
            passwd;
            
            data = new CommData();
            data.setCmd(CommData.CMD_REGISTER_USER);
            data.setData(info);
            
            if (serverConn.connect())
               serverConn.sendData(data);
         }
         
         AddNewUserDialog.this.setVisible(false);
         AddNewUserDialog.this.dispose();
      }
   }
}
