/*
   IM Caf - A Java Based Instant Messaging System
   Author: Bruno Freitas - 08/2004
   Contact: bootsector@ig.com.br
 
   Copyright (c) 2004 Bruno Freitas
 
   Permission is hereby granted, free of charge, to any person obtaining a copy of
   this software and associated documentation files (the "Software"), to deal in
   the Software without restriction, including without limitation the rights to
   use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
   of the Software, and to permit persons to whom the Software is furnished to do
   so, subject to the following conditions:
 
   The above copyright notice and this permission notice shall be included in all
   copies or substantial portions of the Software.
 
   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
   SOFTWARE.
 */

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.net.*;

public class MessageWindow extends JFrame {
   private JTextArea contactArea, myArea;
   private JPanel textPanel, buttonPanel;
   private JButton sendButton, closeButton;
   private JScrollPane csp;
   ServerConnection sc;
   ContactListItem cli;
   
   public MessageWindow(String caption, ServerConnection sc, ContactListItem cli) {
      super(caption);
      
      this.sc = sc;
      this.cli = cli;
      
      textPanel = new JPanel(new GridLayout(2, 1, 0, 3));
      
      contactArea = new JTextArea();
      contactArea.setLineWrap(true);
      contactArea.setWrapStyleWord(true);
      contactArea.setEditable(false);
      //contactArea.setFocusable(false);
      
      myArea = new JTextArea();
      myArea.setLineWrap(true);
      myArea.setWrapStyleWord(true);
      myArea.addKeyListener(new MyKeyListener()); // TAB and SHIFT-TAB and CTRL-ENTER Handler
      
      // Prevent ScrollPanes from getting Focus
      JScrollPane sp = new JScrollPane(contactArea);
      csp = sp;
      sp.getHorizontalScrollBar().setFocusable(false);
      sp.getVerticalScrollBar().setFocusable(false);
      textPanel.add(sp);
      
      sp = new JScrollPane(myArea);
      sp.getHorizontalScrollBar().setFocusable(false);
      sp.getVerticalScrollBar().setFocusable(false);
      textPanel.add(sp);
      
      buttonPanel = new JPanel(new GridLayout(1, 2));
      
      sendButton = new JButton("Send");
      sendButton.setMnemonic('S');
      sendButton.setIcon(new ImageIcon(getClass().getResource("res/img/sendmessage.png")));
      sendButton.addActionListener(
      new ActionListener() {
         public void actionPerformed(ActionEvent e) {
            sendMessage();
         }
      }
      );
      
      closeButton = new JButton("Close");
      closeButton.setMnemonic('C');
      closeButton.setIcon(new ImageIcon(getClass().getResource("res/img/closewindow.png")));
      closeButton.addActionListener(
      new ActionListener() {
         public void actionPerformed(ActionEvent e) {
            updateContactStatus();
            MessageWindow.this.setVisible(false);
         }
      }
      );
      
      buttonPanel.add(sendButton);
      buttonPanel.add(closeButton);
      
      getContentPane().add(textPanel, BorderLayout.CENTER);
      getContentPane().add(buttonPanel, BorderLayout.SOUTH);
      
      // Sets the focus to myArea when window is activated
      addWindowListener(
      new WindowAdapter() {
         public void windowActivated(WindowEvent e) {
            myArea.requestFocus();
         }
         
         public void windowClosing(WindowEvent e) {
            updateContactStatus();
         }
      }
      );
      
      setSize(450, 350);
      setIconImage(new ImageIcon(getClass().getResource("res/img/openletter.png")).getImage());
      //setResizable(false);
      FrameUtils.centerFrameOnScreen(this);
   }
   
   private void sendMessage() {
      if (myArea.getText().length() == 0) return; // Do not send empty messages!
      
      CommData data = new CommData();
      
      data.setCmd(CommData.CMD_INSTANT_MESSAGE);
      data.setFrom(MessageWindow.this.sc.getUserID());
      data.setTo(MessageWindow.this.cli.getID());
      data.setData(myArea.getText());
      
      //contactArea.append("You wrote:\n" + myArea.getText() + "\n\n");
      addMessage("You wrote:\n" + myArea.getText() + "\n\n");
      myArea.setText(null);
      
      MessageWindow.this.sc.sendData(data);
      
      myArea.requestFocus();
      //MessageWindow.this.setVisible(false);
      return;
   }
   
   private void updateContactStatus() {
      CommData data = new CommData();
      
      data.setCmd(CommData.CMD_REQUEST_USER_STATUS);
      data.setTo(cli.getID());
      
      sc.sendData(data);
   }
   
   private class MyKeyListener extends KeyAdapter {
      public void keyPressed(KeyEvent e) {
         if (e.getKeyCode() == KeyEvent.VK_TAB && e.isShiftDown()) {
            ((Component)e.getSource()).transferFocusBackward();
            e.consume();
         }
         else if (e.getKeyCode() == KeyEvent.VK_TAB) {
            ((Component)e.getSource()).transferFocus();
            e.consume();
         }
         else if (e.getKeyCode() == KeyEvent.VK_ENTER && e.isControlDown()) {
            MessageWindow.this.sendMessage();
            e.consume();
         }
      }
   }
   
   public void addMessage(String message) {
      contactArea.append(message);
      contactArea.setCaretPosition(contactArea.getText().length());
      this.setVisible(true);
      
      int state = this.getExtendedState();
      state &= ~Frame.ICONIFIED;
      this.setExtendedState(state);
      
      myArea.requestFocus();
   }
}

