/* 
   IM Caf - A Java Based Instant Messaging System
   Author: Bruno Freitas - 08/2004
   Contact: bootsector@ig.com.br
 
   Copyright (c) 2004 Bruno Freitas

   Permission is hereby granted, free of charge, to any person obtaining a copy of
   this software and associated documentation files (the "Software"), to deal in 
   the Software without restriction, including without limitation the rights to 
   use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies 
   of the Software, and to permit persons to whom the Software is furnished to do 
   so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in all 
   copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, 
   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE 
   SOFTWARE.
*/

import java.net.*;
import java.io.*;
import java.util.*;

public class IMCafeServer {
   public static void main(String args[]) {
      DBInterface dbInterface = null;
      ConnectionsList connList;
      ServerSocket srvSock = null;
      
      System.out.println("IM Cafe - The Server v1.0d\nCopyright(c) 08/2004 - Bruno Freitas - bootsector@ig.com.br");
      System.out.println("JayBird 1.0.1 FireBird JDBC Driver already embedded in this server.\n");
      
      if (args.length != 5) {
         System.out.println("Usage: java -jar IMCafeServer.jar <db_server_name> <db_file_path> <db_user> <db_pass> <im_server_port>");
         System.exit(-1);
      }
      
      System.out.println("Starting server program...");
      
      try {
         dbInterface = new DBInterface(args[0], args[1], args[2], args[3]);
      } catch (Exception e) {
         //throw new RuntimeException(e);
         System.out.println("Could not connect to database server. Operation aborted!");
         System.exit(-1);
      }
      
      connList = new ConnectionsList();
      
      try {
         srvSock = new ServerSocket(Integer.parseInt(args[4]));
      } catch (Exception e) {
         System.out.println("Could not create server socket. Operation aborted!");
         System.exit(-1);
      }
      
      Socket clntConn = null;
      
      System.out.println("Server is up and running!");
      
      while (true) {
         try {
            clntConn = srvSock.accept();
         } catch (Exception e) {
            System.out.println("Error accepting client connection. Operation aborted!");
            System.exit(-1);
         }
         
         try {
            ConnectionItem ci = new ConnectionItem(clntConn);
            connList.addConnection(ci);
            new ClientReader(ci, dbInterface, connList).start();
         } catch (Exception e) {
            // The line below was stopping server if received trash!
            //throw new RuntimeException(e);
         }
      }
   }
}
